@@ -1051,6 +1051,8 @@ GuestdDaemon(GuestApp_Dict **pConfDict,       // IN/OUT
    data = ToolsDaemon_Init(pConfDict, EXEC_LOG,
                            GuestdExecuteHalt, pConfDict,
                            GuestdExecuteReboot, pConfDict,
+                           NULL, NULL,
+                           NULL, NULL,
                            NULL, NULL);
    if (!data) {
       Warning("Unable to start guestd.\n");
@@ -1343,6 +1345,15 @@ main(int argc,    // IN: Number of command line arguments
    int argumentIndex;
    int expectedArgumentIndex;
 
+   if (!VmCheck_IsVirtualWorld()) {
+#ifndef ALLOW_TOOLS_IN_FOREIGN_VM
+      Warning("The VMware service must be run from within a virtual machine.\n");
+      return 0;
+#else
+      runningInForeignVM = TRUE;
+#endif
+   }
+
    confDict = Conf_Load();
 
    /*
@@ -1460,14 +1471,6 @@ main(int argc,    // IN: Number of command line arguments
       GuestdUsage(argv[0], 0);
    }
 
-   if (!VmCheck_IsVirtualWorld()) {
-#ifndef ALLOW_TOOLS_IN_FOREIGN_VM
-      Panic("The VMware service must be run from within a virtual machine.\n");
-#else
-      runningInForeignVM = TRUE;
-#endif
-   }
-
    if (rpci) {
       return GuestdCommandLineRpci(arguments[0]) ? 0 : 1;
    }
