@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -36,7 +36,6 @@
 #error "Gtk 2.0 is required"
 #endif
 
-#include <libgen.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -101,17 +100,14 @@ static void
 AppUtilCollectNamedIcons(GPtrArray *pixbufs,   // IN/OUT
                          const char *iconName) // IN
 {
-   char *myIconName = NULL;
-   char *baseIconName;
+   char *myIconName;
    /*
     * Use the GtkIconTheme to track down any available icons for this app.
     */
    GtkIconTheme *iconTheme = NULL;
    char *ctmp2;
-   Bool foundItInTheme = FALSE; // Did we find this icon in the GtkIconTheme?
-   Bool foundItInFile = FALSE;  // Did we find this icon directly in a file?
+   Bool foundIt; // Did we find this icon in the GtkIconTheme?
    static const char *extraIconPaths[] = {
-      "/usr/share/icons",
       "/usr/share/pixmaps",
       "/usr/local/share/pixmaps",
       "/usr/local/share/icons",
@@ -126,14 +122,12 @@ AppUtilCollectNamedIcons(GPtrArray *pixbufs,   // IN/OUT
 
    ASSERT(iconName);
 
-   Debug("Collecting icons named %s\n", iconName);
-
    iconNameLen = strlen(iconName) + 1;
-   baseIconName = g_alloca(iconNameLen); // We need to modify the name sometimes
-   Str_Strcpy(baseIconName, iconName, iconNameLen);
+   myIconName = g_alloca(iconNameLen); // We need to modify the name sometimes
+   Str_Strcpy(myIconName, iconName, iconNameLen);
 
-   ctmp2 = strrchr(baseIconName, '.');
-   if (*baseIconName != '/' && ctmp2 && strlen(ctmp2) <= 5) {
+   ctmp2 = strrchr(myIconName, '.');
+   if (*myIconName != '/' && ctmp2 && strlen(ctmp2) <= 5) {
       /*
        * If it's a plain filename that we could possibly feed into GtkIconTheme as an
        * icon ID, trim the file extension to turn it into an icon ID string and make
@@ -144,121 +138,8 @@ AppUtilCollectNamedIcons(GPtrArray *pixbufs,   // IN/OUT
 
    iconTheme = gtk_icon_theme_get_default();
    g_object_ref(G_OBJECT(iconTheme));
-   foundItInTheme = AppUtilIconThemeReallyHasIcon(iconTheme, baseIconName);
-
-   if (!foundItInTheme) {
-      /*
-       * Try looking for it as a non-GtkIconTheme managed file, to deal with older
-       * systems.
-       */
-      if (iconName[0] != '/') {
-         gchar *iconSearchName;
-         gsize iconSearchNameSize;
-         gboolean triedOtherExts = FALSE;
-
-         char *ctmp2;
-         int i;
-
-         static const char *iconExtensions[] = {
-            "",         // MUST be first entry.
-            ".png",
-            ".xpm",
-            ".gif",
-            ".svg",
-         };
-
-         /*
-          * Make a local copy of iconName in case we need to search for icons with
-          * alternate extensions.
-          */
-         iconSearchNameSize = strlen(iconName) + 1;
-         iconSearchName = g_alloca(iconSearchNameSize);
-         g_strlcpy(iconSearchName, iconName, iconSearchNameSize);
-
-         myIconName = NULL;
-
-         ctmp2 = g_alloca(PATH_MAX);
-tryingOtherExts:
-         for (i = 0; !myIconName && i < ARRAYSIZE(extraIconPaths); i++) {
-            int j;
-
-            if (!extraIconPaths[i]) {
-               continue;
-            }
-
-            /*
-             * Per Desktop Entry Specification and Icon Theme Specification, I don't
-             * believe that the iconName, unless it's an absolute path, should include
-             * the file extension.
-             *
-             * However, empirical evidence shows that -many- applications ignore that
-             * and may specify an icon of, for example, "foo.png".  We'll also handle
-             * those special cases here.
-             */
-            for (j = 0; !myIconName && j < ARRAYSIZE(iconExtensions); j++) {
-               g_snprintf(ctmp2, PATH_MAX, "%s/%s%s", extraIconPaths[i], iconSearchName,
-                          iconExtensions[j]);
-               if (!g_file_test(ctmp2, G_FILE_TEST_EXISTS)) {
-                  continue;
-               }
-
-               if (j != 0) {
-                  /*
-                   * Case 1:  We located an icon by appending an image extension to
-                   * IconName.  Success!
-                   */
-                  myIconName = ctmp2;           // Will break "i" loop.
-                  foundItInFile = TRUE;
-                  break;
-               } else {
-                  /*
-                   * Case 2:  We found an icon without appending an extension.  Verify
-                   * that the filename contains a valid image extension.
-                   */
-                  int k;
-                  char *ctmpext = strrchr(ctmp2, '.');
-                  for (k = 1; ctmpext && k < ARRAYSIZE(iconExtensions); k++) {
-                     if (strcmp(ctmpext, iconExtensions[k]) == 0) {
-                        myIconName = ctmp2;     // Will break "i" loop.
-                        foundItInFile = TRUE;
-                        break;
-                     }
-                  }
-               }
-            }
-         }
-
-         /*
-          * No dice.  But we won't give up hope just yet!  In some cases, icon-foo.png
-          * may not exist while icon-foo.xpm does.  (Ex: Ubuntu 8.04's hwtest.)  We'll
-          * try once more by searching for an icon with an alternate extension.
-          */
-         if (!foundItInFile && !triedOtherExts) {
-            int extIndex;
-
-            triedOtherExts = TRUE;
-
-            /*
-             * Look to see if iconSearchName contains any of the known extensions
-             * listed above.  If so, chop it off, then reattempt the search.
-             */
-            for (extIndex = 1; extIndex < ARRAYSIZE(iconExtensions); extIndex++) {
-               if (g_str_has_suffix(iconSearchName, iconExtensions[extIndex])) {
-                  gchar *extension;
-                  extension = g_strrstr(iconSearchName, iconExtensions[extIndex]);
-                  *extension = '\0';
-                  goto tryingOtherExts;
-               }
-            }
-         }
-      } else {
-         myIconName = g_alloca(iconNameLen + 1);
-         Str_Strcpy(myIconName, iconName, iconNameLen);
-         foundItInFile = TRUE;
-      }
-   }
-
-   if (!foundItInTheme && !foundItInFile) {
+   foundIt = AppUtilIconThemeReallyHasIcon(iconTheme, myIconName);
+   if (!foundIt) {
       /*
        * Try looking through some auxiliary icon themes.
        */
@@ -271,8 +152,6 @@ tryingOtherExts:
          "Bluecurve",
          "HighContrast-SVG",
          "HighContrastLargePrint",
-         "crystalsvg",
-         "slick",
          NULL
       };
       g_object_unref(G_OBJECT(iconTheme));
@@ -289,14 +168,62 @@ tryingOtherExts:
 
       for (i = 0; extraThemes[i]; i++) {
          gtk_icon_theme_set_custom_theme(iconTheme, extraThemes[i]);
-         foundItInTheme = AppUtilIconThemeReallyHasIcon(iconTheme, baseIconName);
-         if (foundItInTheme) {
+         foundIt = AppUtilIconThemeReallyHasIcon(iconTheme, myIconName);
+         if (foundIt) {
             break;
          }
       }
    }
 
-   if (foundItInTheme) {
+   if (!foundIt) {
+      /*
+       * Try looking for it as a non-GtkIconTheme managed file, to deal with older
+       * systems.
+       */
+      if (iconName[0] != '/') {
+         char *ctmp2;
+         char *ctmpext;
+         int i;
+
+         myIconName = NULL;
+         ctmpext = strrchr(iconName, '.');
+         ctmp2 = g_alloca(PATH_MAX);
+         for (i = 0; !myIconName && i < ARRAYSIZE(extraIconPaths); i++) {
+            if (!extraIconPaths[i]) {
+               continue;
+            }
+
+            if (ctmpext) {
+               g_snprintf(ctmp2, PATH_MAX, "%s/%s", extraIconPaths[i], iconName);
+               if (g_file_test(ctmp2, G_FILE_TEST_EXISTS)) {
+                  myIconName = ctmp2;
+               }
+            } else {
+               static const char *iconExtensions[] = {
+                  ".png",
+                  ".xpm",
+                  ".gif",
+                  ".svg",
+               };
+               int j;
+
+               for (j = 0; j < ARRAYSIZE(iconExtensions); j++) {
+                  g_snprintf(ctmp2, PATH_MAX, "%s/%s%s",
+                             extraIconPaths[i],
+                             iconName, iconExtensions[j]);
+                  if (g_file_test(ctmp2, G_FILE_TEST_EXISTS)) {
+                     myIconName = ctmp2;
+                     break;
+                  }
+               }
+            }
+         }
+      } else {
+         Str_Strcpy(myIconName, iconName, iconNameLen);
+      }
+   }
+
+   if (foundIt) {
       /*
        * If we know this icon can be loaded via GtkIconTheme, do so.
        */
@@ -304,8 +231,7 @@ tryingOtherExts:
       int i;
       Bool needToUseScalable;
 
-      iconSizes = gtk_icon_theme_get_icon_sizes(iconTheme, baseIconName);
-      Debug("Loading icon %s from iconTheme\n", baseIconName);
+      iconSizes = gtk_icon_theme_get_icon_sizes(iconTheme, myIconName);
 
       ASSERT(iconSizes);
       needToUseScalable = (iconSizes[0] == -1 && iconSizes[1] == 0);
@@ -328,10 +254,10 @@ tryingOtherExts:
             thisSize = 64; // Render SVG icons to 64x64
          }
 
-         iconInfo = gtk_icon_theme_lookup_icon(iconTheme, baseIconName, thisSize, 0);
+         iconInfo = gtk_icon_theme_lookup_icon(iconTheme, myIconName, thisSize, 0);
 
          if (!iconInfo) {
-            Debug("Couldn't find %s icon for size %d\n", baseIconName, thisSize);
+            Debug("Couldn't find %s icon for size %d\n", myIconName, thisSize);
             continue;
          }
 
@@ -344,7 +270,7 @@ tryingOtherExts:
          if (pixbuf) {
             g_ptr_array_add(pixbufs, pixbuf);
          } else {
-            Debug("WARNING: Not even a built-in pixbuf for icon %s\n", baseIconName);
+            Debug("WARNING: Not even a built-in pixbuf for icon %s\n", myIconName);
          }
 
          gtk_icon_info_free(iconInfo);
@@ -352,9 +278,8 @@ tryingOtherExts:
 
       g_free(iconSizes);
 
-   } else if (foundItInFile && myIconName && myIconName[0] == '/') {
+   } else if (myIconName && myIconName[0] == '/') {
       GdkPixbuf *pixbuf;
-      Debug("Loading icon %s from file\n", myIconName);
       pixbuf = gdk_pixbuf_new_from_file(myIconName, NULL);
       if (pixbuf) {
          g_ptr_array_add(pixbufs, pixbuf);
@@ -442,6 +367,8 @@ AppUtil_CollectIconArray(const char *iconName,        // IN
 {
    GPtrArray *pixbufs;
 
+   ASSERT(iconName != NULL || windowID != None);
+
    pixbufs = g_ptr_array_new();
 
    if (iconName) {
@@ -728,8 +655,7 @@ AppUtil_AppIsSkippable(const char *appName)
       "python2.3",
       "python2.2",
       "perl",
-      "sh",
-      "bash",
+      "getproxy"
    };
    char cbuf[PATH_MAX];
    int i;
@@ -772,33 +698,26 @@ AppUtil_CanonicalizeAppName(const char *appName, // IN
    ASSERT(appName);
 
    if (appName[0] == '/') {
-      ctmp = g_strdup(appName);
-      goto exit;
+      return g_strdup(appName);
    }
 
    ctmp = g_find_program_in_path(appName);
    if (ctmp) {
-      goto exit;
+      return ctmp;
    }
 
-   /*
-    * It's probably safe to assume that cwd is an absolute path (at the time of
-    * writing, it is derived from /proc), but let's check to be sure.
-    */
-   if (cwd && cwd[0] == '/') {
+   if (cwd) {
+      char cbuf[PATH_MAX];
 
-      /* Don't add any unnecessary path separators. */
-      char *cbuf = Str_Asprintf(NULL, "%s%s%s", cwd,
-                                cwd[strlen(cwd) - 1] == '/' ? "" : "/",
-                                appName);
-      if (cbuf) {
-         ctmp = Posix_RealPath(cbuf);
-         free(cbuf);
-      }
+      getcwd(cbuf, sizeof cbuf);
+      chdir(cwd);
+      ctmp = Posix_RealPath(appName);
+      chdir(cbuf);
+
+      return ctmp;
    }
 
- exit:
-   return ctmp;
+   return NULL;
 }
 
 
