@@ -395,8 +395,8 @@ GetShortName(char *distro,             // IN: full distro name
       Str_Strcpy(distroShort, STR_OS_LINUX_FROM_SCRATCH, distroShortSize);
    } else if (strstr(distroLower, "linux-ppc")) {
       Str_Strcpy(distroShort, STR_OS_LINUX_PPC, distroShortSize);
-   } else if (strstr(distroLower, "mandravia")) {
-      Str_Strcpy(distroShort, STR_OS_MANDRAVIA, distroShortSize);
+   } else if (strstr(distroLower, "mandriva")) {
+      Str_Strcpy(distroShort, STR_OS_MANDRIVA, distroShortSize);
    } else if (strstr(distroLower, "mklinux")) {
       Str_Strcpy(distroShort, STR_OS_MKLINUX, distroShortSize);
    } else if (strstr(distroLower, "pld")) {
