@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,20 +16,6 @@
  *
  *********************************************************/
 
-/*********************************************************
- * The contents of this file are subject to the terms of the Common
- * Development and Distribution License (the "License") version 1.0
- * and no later version.  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the License at
- *         http://www.opensource.org/licenses/cddl1.php
- *
- * See the License for the specific language governing permissions
- * and limitations under the License.
- *
- *********************************************************/
-
 /*
  * cpName.c --
  *
@@ -45,12 +31,11 @@
 #include "cpName.h"
 #include "cpNameInt.h"
 #include "vm_assert.h"
-#include "hgfsEscape.h"
 
 /*
  *----------------------------------------------------------------------
  *
- * CPName_GetComponent --
+ * CPName_GetComponentGeneric --
  *
  *    Get the next component of the CP name.
  *
@@ -59,6 +44,9 @@
  *    any. The "next" pointer is set to "end" if there is no next
  *    component.
  *
+ *    'illegal' is a string of characters that are not allowed to
+ *    be present in the pre-converted CP name.
+ *
  * Results:
  *    length (not including NUL termination) >= 0 of next
  *    component on success.
@@ -71,9 +59,10 @@
  */
 
 int
-CPName_GetComponent(char const *begin,   // IN: Beginning of buffer
-                    char const *end,     // IN: End of buffer
-                    char const **next)   // OUT: Start of next component
+CPName_GetComponentGeneric(char const *begin,   // IN: Beginning of buffer
+                           char const *end,     // IN: End of buffer
+                           char const *illegal, // IN: Illegal characters
+                           char const **next)   // OUT: Start of next component
 {
    char const *walk;
    char const *myNext;
@@ -82,6 +71,7 @@ CPName_GetComponent(char const *begin,   // IN: Beginning of buffer
    ASSERT(begin);
    ASSERT(end);
    ASSERT(next);
+   ASSERT(illegal);
    ASSERT(begin <= end);
 
    for (walk = begin; ; walk++) {
@@ -96,86 +86,57 @@ CPName_GetComponent(char const *begin,   // IN: Beginning of buffer
          /* Found a NUL */
 
          if (walk == begin) {
-            Log("%s: error: first char can't be NUL\n", __FUNCTION__);
+            Log("CPName_GetComponentGeneric: error: first char can't be NUL\n");
             return -1;
          }
 
          myNext = walk + 1;
-         /* Skip consecutive path delimiters. */
-         while ((*myNext == '\0') && (myNext != end)) {
-            myNext++;
-         }
          if (myNext == end) {
             /* Last character in the buffer is not allowed to be NUL */
-            Log("%s: error: last char can't be NUL\n", __FUNCTION__);
             return -1;
          }
 
          break;
       }
+
+      /*
+       * Make sure the input buffer does not contain any illegal
+       * characters. In particular, we want to make sure that there
+       * are no path separator characters in the name. Since the
+       * cross-platform name format by definition does not use path
+       * separators, this is an error condition, and is likely the
+       * sign of an attack. See bug 27926. [bac]
+       *
+       * The test above ensures that *walk != NUL here, so we don't
+       * need to test it again before calling strchr().
+       */
+      if (strchr(illegal, *walk) != NULL) {
+         Log("CPName_GetComponentGeneric: error: Illegal char \"%c\" found in "
+             "input\n", *walk);
+         return -1;
+      }
    }
 
    len = walk - begin;
 
-   *next = myNext;
-   return (int) len;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * CPNameEscapeAndConvertFrom --
- *
- *    Converts a cross-platform name representation into a string for
- *    use in the local filesystem.
- *    Escapes illegal characters as a part of convertion.
- *    This is a cross-platform implementation and takes the path separator
- *    argument as an argument. The path separator is prepended before each
- *    additional path component, so this function never adds a trailing path
- *    separator.
- *
- * Results:
- *    0 on success.
- *    error < 0 on failure (the converted string did not fit in
- *    the buffer provided or the input was invalid).
- *
- * Side effects:
- *    None
- *
- *----------------------------------------------------------------------
- */
-
-int
-CPNameEscapeAndConvertFrom(char const **bufIn, // IN/OUT: Input to convert
-                           size_t *inSize,     // IN/OUT: Size of input
-                           size_t *outSize,    // IN/OUT: Size of output buffer
-                           char **bufOut,      // IN/OUT: Output buffer
-                           char pathSep)       // IN: Path separator character
-{
-   int result;
-   int inputSize;
-   inputSize = HgfsEscape_GetSize(*bufIn, *inSize);
-   if (inputSize < 0) {
-      result = -1;
-   } else if (inputSize != 0) {
-      char *savedBufOut = *bufOut;
-      char const *savedOutConst = savedBufOut;
-      size_t savedOutSize = *outSize;
-      if (inputSize > *outSize) {
-         Log("%s: error: not enough room for escaping\n", __FUNCTION__);
-         return -1;
-      }
-
-      /* Leaving space for the leading path separator, thus output to savedBufOut + 1. */
-      *inSize = HgfsEscape_Do(*bufIn, *inSize, savedOutSize, savedBufOut + 1);
-      result = CPNameConvertFrom(&savedOutConst, inSize, outSize, bufOut, pathSep);
-      *bufIn += *inSize;
-      *inSize = 0;
-   } else {
-      result = CPNameConvertFrom(bufIn, inSize, outSize, bufOut, pathSep);
+   /* 
+    * We're only interested in looking for dot/dotdot if the illegal character
+    * string isn't empty. These characters are only relevant when the resulting
+    * string is to be passed down to the filesystem. Some callers (such as the
+    * HGFS server, when dealing with actual filenames) do care about this 
+    * validation, but others (like DnD, hgFileCopy, and the HGFS server when
+    * converting share names) just want to convert a CPName down to a 
+    * nul-terminated string. 
+    */
+   if (strcmp(illegal, "") != 0 &&
+       ((len == 1 && memcmp(begin, ".", 1) == 0) ||
+        (len == 2 && memcmp(begin, "..", 2) == 0))) {
+      Log("CPName_GetComponentGeneric: error: found dot/dotdot\n");
+      return -1;
    }
-   return result;
+
+   *next = myNext;
+   return ((int) len);
 }
 
 
@@ -213,7 +174,6 @@ CPNameConvertFrom(char const **bufIn, // IN/OUT: Input to convert
    char const *inEnd;
    size_t myOutSize;
    char *out;
-   Bool inPlaceConvertion = (*bufIn == *bufOut);
 
    ASSERT(bufIn);
    ASSERT(inSize);
@@ -221,9 +181,6 @@ CPNameConvertFrom(char const **bufIn, // IN/OUT: Input to convert
    ASSERT(bufOut);
 
    in = *bufIn;
-   if (inPlaceConvertion) {
-      in++; // Skip place for the leading path separator.
-   }
    inEnd = in + *inSize;
    myOutSize = *outSize;
    out = *bufOut;
@@ -235,17 +192,10 @@ CPNameConvertFrom(char const **bufIn, // IN/OUT: Input to convert
 
       len = CPName_GetComponent(in, inEnd, &next);
       if (len < 0) {
-         Log("%s: error: get next component failed\n", __FUNCTION__);
+         Log("CPNameConvertFrom: error: get next component failed\n");
          return len;
       }
 
-      /* Bug 27926 - preventing escaping from shared folder. */
-      if ((len == 1 && *in == '.') ||
-          (len == 2 && in[0] == '.' && in[1] == '.')) {
-         Log("%s: error: found dot/dotdot\n", __FUNCTION__);
-         return -1;
-      }
-
       if (len == 0) {
          /* No more component */
          break;
@@ -253,15 +203,13 @@ CPNameConvertFrom(char const **bufIn, // IN/OUT: Input to convert
 
       newLen = ((int) myOutSize) - len - 1;
       if (newLen < 0) {
-         Log("%s: error: not enough room\n", __FUNCTION__);
+         Log("CPNameConvertFrom: error: not enough room\n");
          return -1;
       }
       myOutSize = (size_t) newLen;
 
       *out++ = pathSep;
-      if (!inPlaceConvertion) {
-         memcpy(out, in, len);
-      }
+      memcpy(out, in, len);
       out += len;
 
       in = next;
@@ -269,7 +217,7 @@ CPNameConvertFrom(char const **bufIn, // IN/OUT: Input to convert
 
    /* NUL terminate */
    if (myOutSize < 1) {
-      Log("%s: error: not enough room\n", __FUNCTION__);
+      Log("CPNameConvertFrom: error: not enough room\n");
       return -1;
    }
    *out = '\0';
@@ -360,7 +308,7 @@ CPName_LinuxConvertTo(char const *nameIn, // IN:  Buf to convert
                       size_t bufOutSize,  // IN:  Size of the output buffer
                       char *bufOut)       // OUT: Output buffer
 {
-   return CPNameConvertTo(nameIn, bufOutSize, bufOut, '/');
+   return CPNameConvertTo(nameIn, bufOutSize, bufOut, '/', NULL);
 }
 
 
@@ -391,7 +339,7 @@ CPName_WindowsConvertTo(char const *nameIn, // IN:  Buf to convert
                         size_t bufOutSize,  // IN:  Size of the output buffer
                         char *bufOut)       // OUT: Output buffer
 {
-   return CPNameConvertTo(nameIn, bufOutSize, bufOut, '\\');
+   return CPNameConvertTo(nameIn, bufOutSize, bufOut, '\\', ":");
 }
 
 
@@ -402,10 +350,6 @@ CPName_WindowsConvertTo(char const *nameIn, // IN:  Buf to convert
  *
  *    Makes a cross-platform name representation from the input string
  *    and writes it into the output buffer.
- *    HGFS convention is to echange names between guest and host in uescaped form.
- *    Both ends perform necessary name escaping according to its own rules
- *    to avoid presenitng invalid file names to OS. Thus the name needs to be unescaped
- *    as a part of conversion to host-independent format.
  *
  * Results:
  *    On success, returns the number of bytes used in the
@@ -422,9 +366,10 @@ int
 CPNameConvertTo(char const *nameIn, // IN:  Buf to convert
                 size_t bufOutSize,  // IN:  Size of the output buffer
                 char *bufOut,       // OUT: Output buffer
-                char pathSep)       // IN:  path separator to use
+                char pathSep,       // IN:  path separator to use
+                char *ignores)      // IN:  chars to not transfer to output
 {
-   char *origOut = bufOut;
+   char const *origOut = bufOut;
    char const *endOut = bufOut + bufOutSize;
    size_t cpNameLength = 0;
 
@@ -436,22 +381,31 @@ CPNameConvertTo(char const *nameIn, // IN:  Buf to convert
       nameIn++;
    }
 
-    /*
-     * Copy the string to the output buf, converting all path separators into '\0'.
-     * Collapse multiple consecutive path separators into a single one since
-     * CPName_GetComponent can't handle consecutive path separators.
-     */
-   while (*nameIn != '\0' && bufOut < endOut) {
-      if (*nameIn == pathSep) {
-         *bufOut = '\0';
-         do {
-            nameIn++;
-         } while (*nameIn == pathSep);
+   /*
+    * Copy the string to the output buf, converting all path separators into
+    * '\0' and ignoring the specified characters.
+    */
+   for (; *nameIn != '\0' && bufOut < endOut; nameIn++) {
+      if (ignores) {
+         char *currIgnore = ignores;
+         Bool ignore = FALSE;
+
+         while (*currIgnore != '\0') {
+            if (*nameIn == *currIgnore) {
+               ignore = TRUE;
+               break;
+            }
+            currIgnore++;
+         }
+
+         if (!ignore) {
+            *bufOut = (*nameIn == pathSep) ? '\0' : *nameIn;
+            bufOut++;
+         }
       } else {
-         *bufOut = *nameIn;
-         nameIn++;
+         *bufOut = (*nameIn == pathSep) ? '\0' : *nameIn;
+         bufOut++;
       }
-      bufOut++;
    }
 
    /*
@@ -473,7 +427,6 @@ CPNameConvertTo(char const *nameIn, // IN:  Buf to convert
    while ((cpNameLength >= 1) && (origOut[cpNameLength - 1] == 0)) {
       cpNameLength--;
    }
-   cpNameLength = HgfsEscape_Undo(origOut, cpNameLength);
 
    /* Return number of bytes used */
    return (int) cpNameLength;
