@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2016 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,5 +30,10 @@
 #   include "vm_basic_types.h"
 
 void Debug(char const *fmt, ...) PRINTF_DECL(1, 2);
+void Debug_Set(Bool enable, const char *prefix);
+Bool Debug_IsEnabled(void);
+void Debug_EnableToFile(const char *file, Bool backup);
+Bool Debug_EnableIfFile(const char *enableFile, const char *toFileFile);
+
 
 #endif /* __DEBUG_H__ */
