@@ -224,7 +224,12 @@ Posix_GetHostName(Unicode name, // OUT
  *
  * Posix_GetHostByName --
  *
- *      Wrapper for gethostbyname().
+ *      Wrapper for gethostbyname().  Caller should release memory
+ *      allocated for the hostent structure returned by calling
+ *      Posix_FreeHostent().
+ *      
+ *      TODO: Implement this wrapper and Posix_FreeHostent() for the
+ *      non-Windows platforms.
  *
  * Results:
  *      NULL    Error
@@ -239,73 +244,33 @@ Posix_GetHostName(Unicode name, // OUT
 static INLINE struct hostent*
 Posix_GetHostByName(ConstUnicode name)  // IN
 {
-   static Atomic_uint32 atomicIndex = {TLS_OUT_OF_INDEXES};
-   uint32 hostentIndex;
-   struct hostent *shostent;
+   struct hostent *newhostent;
    char *nameMBCS;
    struct hostent *hostentMBCS;
    struct hostent *ret = NULL;
 
    ASSERT(name);
 
-   /*
-    * To make this wrapper thread-safe, we need to allocate one struct hostent
-    * per thread.  The pointer to that structure is stored in TLS.  The TLS
-    * index is allocated the first time this wrapper is called (in a process).
-    * We use atomic operations to make sure we only allocate one TLS index.
-    */
-
-   if ((hostentIndex = Atomic_Read(&atomicIndex)) == TLS_OUT_OF_INDEXES) {
-      uint32 newIndex = TlsAlloc();
-
-      ASSERT_NOT_IMPLEMENTED(newIndex != TLS_OUT_OF_INDEXES);
-      Atomic_Init();
-      hostentIndex = Atomic_ReadIfEqualWrite(&atomicIndex, TLS_OUT_OF_INDEXES, newIndex);
-      if (hostentIndex == TLS_OUT_OF_INDEXES) {
-         hostentIndex = newIndex;
-      } else {
-         TlsFree(newIndex);
-      }
-   }
-
    nameMBCS = (char *)Unicode_GetAllocBytes(name, STRING_ENCODING_DEFAULT);
 
    if (nameMBCS != NULL) {
       hostentMBCS = gethostbyname(nameMBCS);
 
       if (hostentMBCS != NULL) {
-         shostent = (struct hostent *)TlsGetValue(hostentIndex);
-         if (shostent == NULL) {
-            /*
-             * Allocate the structure the first time this wrapper is called
-             * within a thread and re-use it afterwards.
-             */
+         newhostent = (struct hostent *)Util_SafeMalloc(sizeof *newhostent);
 
-            BOOL retTLS;
-
-            shostent = (struct hostent *)Util_SafeMalloc(sizeof *shostent);
-            retTLS = TlsSetValue(hostentIndex, shostent);
-            ASSERT_NOT_IMPLEMENTED(retTLS);
-            shostent->h_name = NULL;
-            shostent->h_aliases = NULL;
-         }
-
-         Unicode_Free(shostent->h_name);
-         if (shostent->h_aliases) {
-            Unicode_FreeList(shostent->h_aliases, -1);
-            shostent->h_aliases = NULL;
-         }
-
-         shostent->h_name = Unicode_Alloc(hostentMBCS->h_name,
-                                          STRING_ENCODING_DEFAULT);
+         newhostent->h_name = Unicode_Alloc(hostentMBCS->h_name,
+                                            STRING_ENCODING_DEFAULT);
          if (hostentMBCS->h_aliases) {
-            shostent->h_aliases = Unicode_AllocList(hostentMBCS->h_aliases, -1,
-                                                    STRING_ENCODING_DEFAULT);
+            newhostent->h_aliases = Unicode_AllocList(hostentMBCS->h_aliases, -1,
+                                                      STRING_ENCODING_DEFAULT);
+         } else {
+            newhostent->h_aliases = NULL;
          }
-         shostent->h_addrtype = hostentMBCS->h_addrtype;
-         shostent->h_length = hostentMBCS->h_length;
-         shostent->h_addr_list = hostentMBCS->h_addr_list;
-         ret = shostent;
+         newhostent->h_addrtype = hostentMBCS->h_addrtype;
+         newhostent->h_length = hostentMBCS->h_length;
+         newhostent->h_addr_list = hostentMBCS->h_addr_list;
+         ret = newhostent;
       }
    } else {
       /* There has been an error converting from UTF-8 to local encoding. */
@@ -314,6 +279,36 @@ Posix_GetHostByName(ConstUnicode name)  // IN
 
    return ret;
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Posix_FreeHostent --
+ *
+ *      Free the memory allocated for an hostent structure returned
+ *      by Posix_AllocGetHostByName.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      NOne.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static INLINE void
+Posix_FreeHostent(struct hostent *he)
+{
+   if (he) {
+      Unicode_Free(he->h_name);
+      if (he->h_aliases) {
+         Unicode_FreeList(he->h_aliases, -1);
+      }
+      free(he);
+   }
+}
 #endif	// defined(_WINSOCKAPI_) || defined(_WINSOCK2API_)
 
 #ifdef _WS2TCPIP_H_
