@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -60,10 +60,10 @@
  *----------------------------------------------------------------------
  */
 
-char *
+Unicode
 Hostinfo_HostName(void)
 {
-   char *result;
+   Unicode result;
    HMODULE dllHandle;
    struct hostent *myHostEnt;
    struct hostent *(WINAPI *GetHostByNameFn)(char *hostName);
@@ -88,7 +88,6 @@ Hostinfo_HostName(void)
 
       if (!dllHandle) {
          Warning("%s Failed to wsock32.\n", __FUNCTION__);
-
          return NULL;
       }
    }
@@ -98,14 +97,12 @@ Hostinfo_HostName(void)
    if (!GetHostNameFn) {
       Warning("%s Failed to find gethostname.\n", __FUNCTION__);
       FreeLibrary(dllHandle);
-
       return NULL;
    }
 
    if ((*GetHostNameFn)(hostName, sizeof hostName) == SOCKET_ERROR) {
       Warning("%s gethostname failed.\n", __FUNCTION__);
       FreeLibrary(dllHandle);
-
       return NULL;
    }
 
@@ -114,7 +111,6 @@ Hostinfo_HostName(void)
    if (!GetHostByNameFn) {
       Warning("%s Failed to find gethostbyname.\n", __FUNCTION__);
       FreeLibrary(dllHandle);
-
       return Unicode_Alloc(hostName, STRING_ENCODING_DEFAULT);
    }
 
@@ -156,12 +152,12 @@ Hostinfo_HostName(void)
  *-----------------------------------------------------------------------------
  */
 
-char *
+Unicode
 Hostinfo_HostName(void)
 {
    struct utsname un;
 
-   char *result = NULL;
+   Unicode result = NULL;
 
    if ((uname(&un) == 0) && (*un.nodename != '\0')) {
       /* 'un.nodename' is already fully qualified. */
@@ -170,7 +166,7 @@ Hostinfo_HostName(void)
 
    return result;
 }
-#elif defined(linux)
+#elif defined(linux)		// Linux
 #include <unistd.h>
 #include <sys/utsname.h>
 #include <netdb.h>
@@ -192,12 +188,12 @@ Hostinfo_HostName(void)
  *-----------------------------------------------------------------------------
  */
 
-char *
+Unicode
 Hostinfo_HostName(void)
 {
    struct utsname un;
 
-   char *result = NULL;
+   Unicode result = NULL;
 
    if ((uname(&un) == 0) && (*un.nodename != '\0')) {
       char *p;
@@ -231,20 +227,24 @@ Hostinfo_HostName(void)
  *
  * Hostinfo_HostName --
  *
- *      Stub for uncoded OSen
+ *      Return the fully qualified host name of the host.
  *
  * Results:
- *      NULL
+ *      The host name on success; must be freed
+ *      NULL on failure
  *
  * Side effects:
- *      None
+ *       None
+ *
+ * Note:
+ *	This is a dummy catcher for uncoded OSen.
  *
  *-----------------------------------------------------------------------------
  */
 
-char *
+Unicode
 Hostinfo_HostName(void)
 {
-   return NULL;
+   return Unicode_Format("%s: unimplemented for OS", __FUNCTION__);
 }
 #endif
