@@ -17,6 +17,9 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
+ * 4. Neither the name of the University nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -740,17 +743,6 @@ bsd_vsnwprintf(wchar_t **outBuf, size_t bufSize, const wchar_t *fmt0,
          flags |= GROUPING;
          thousands_sep = (wchar_t) *(localeconv()->thousands_sep);
          grouping = localeconv()->grouping;
-
-	 /*
-	  * Grouping should not begin with 0, but it nevertheless
-	  * does (see bug 281072) and makes the formatting code
-	  * behave badly, so we fix it up.
-	  */
-
-	 if (grouping != NULL && *grouping == '\0') {
-	    static char g[] = { CHAR_MAX, '\0' };
-	    grouping = g;
-	 }
          goto rflag;
       case '.':
          if ((ch = *fmt++) == '*') {
@@ -1294,6 +1286,7 @@ bsd_vsnwprintf(wchar_t **outBuf, size_t bufSize, const wchar_t *fmt0,
    int n, n2;                /* handy integer (short term usage) */
    wchar_t *cp;                /* handy char pointer (short term usage) */
    int flags;                /* flags as above */
+   int width;                /* width from format (%8d), or 0 */
    enum typeid *typetable; /* table of types */
    enum typeid stattypetable [STATIC_ARG_TBL_SIZE];
    int tablesize;                /* current size of type table */
@@ -1360,6 +1353,7 @@ bsd_vsnwprintf(wchar_t **outBuf, size_t bufSize, const wchar_t *fmt0,
       fmt++;                /* skip over '%' */
 
       flags = 0;
+      width = 0;
 
      rflag:          ch = *fmt++;
      reswitch:       switch (ch) {
@@ -1395,6 +1389,7 @@ bsd_vsnwprintf(wchar_t **outBuf, size_t bufSize, const wchar_t *fmt0,
             nextarg = n;
             goto rflag;
          }
+         width = n;
          goto reswitch;
       case 'h':
          if (flags & SHORTINT) {
