@@ -61,15 +61,6 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-
-/*
- * Warn builders up front that GTK 2 is mandatory.
- */
-#ifndef GTK2
-#   error UnityX11 depends on GTK+ 2.
-#endif
-
-
 /*
  * These defines are listed in the EWMH spec, but not available in any header file that I
  * know of.
@@ -119,8 +110,6 @@ typedef enum {
    UNITY_X11_WM__NET_WM_ACTION_MAXIMIZE_VERT,
    UNITY_X11_WM__NET_FRAME_EXTENTS,
    UNITY_X11_WM__NET_WM_STRUT_PARTIAL,
-   UNITY_X11_WM__NET_WM_STATE_HIDDEN,
-   UNITY_X11_WM__NET_WM_STATE_MINIMIZED,
 
    UNITY_X11_MAX_WM_PROTOCOLS
 } UnityX11WMProtocol;
@@ -150,38 +139,13 @@ struct UnitySpecialWindow {
 
 typedef struct UnityPlatformWindow UnityPlatformWindow;
 
-
-/**
- * Custom Glib source to monitor the Xlib queue and X11 socket(s).
- *
- * This structure is "derived from" GSource.  It will be recast and used as
- * a GSource with Glib functions.
- */
-
-typedef struct {
-   GSource base;        ///< Owned by Glib.  Hands off!
-   UnityPlatform *up;   ///< Our running UnityPlatform context.
-   GHashTable *fdTable; ///< file descriptor => GPollFD *.
-} UnityGSource;
-
-
-/**
+/*
  * Holds platform-specific data.
  */
 struct _UnityPlatform {
    Display *display; // X11 display object
    long eventTimeDiff; // Diff between X server time and our local time
-   /**
-    * Integrates Unity event sources with the Glib main loop.
-    *
-    * Monitors dedicated Unity X11 socket(s) and the associated Xlib event
-    * queue.  Its lifetime is for the duration that the user is in Unity mode.
-    *
-    * @note
-    * The event source is created in UnityPlatformStartHelperThreads and torn
-    * down in UnityPlatformKillHelperThreads.
-    */
-   UnityGSource *glibSource;
+   guint unityDisplayWatchID; // Manages listening to the display from the glib main loop
 
    struct { // Atoms that we'll find useful
       Atom _NET_WM_WINDOW_TYPE,
@@ -239,18 +203,16 @@ struct _UnityPlatform {
       _NET_SUPPORTED,
       _NET_FRAME_EXTENTS,
       WM_CLASS,
-      WM_CLIENT_LEADER,
       WM_DELETE_WINDOW,
       WM_ICON,
       WM_NAME,
       WM_PROTOCOLS,
       WM_STATE,
-      WM_TRANSIENT_FOR,
-      WM_WINDOW_ROLE;
+      WM_TRANSIENT_FOR;
    } atoms;
 
    UnityWindowTracker *tracker;
-   UnityUpdateChannel *updateChannel;
+   UnityUpdateThreadData updateData;
 
    /*
     * This tracks all toplevel windows, whether or not they are showing through to the
@@ -291,13 +253,6 @@ struct _UnityPlatform {
       Atom savedLayoutData[4];
       size_t savedNumDesktops;
       uint32 savedCurrentDesktop;
-      uint32 currentDesktop;
-      /*
-       * Set by the set.desktop.active RPC to indicate which desktop the host is in
-       * when we enter Unity.
-       */
-      uint32 initialDesktop;
-#define UNITY_X11_INITIALDESKTOP_UNSET  MAX_UINT32
    } desktopInfo;
 
    Bool isRunning;
@@ -334,7 +289,6 @@ struct UnityPlatformWindow {
    Window rootWindow;
    int screenNumber;
    int desktopNumber;
-   int onUnmapDesktopNumber;    ///< @sa wantSetDesktopNumberOnUnmap
    UnityPlatformWindow *higherWindow;
    UnityPlatformWindow *lowerWindow;
 
@@ -346,6 +300,7 @@ struct UnityPlatformWindow {
       UnityIconType type;
    } iconPng;
 
+   XConfigureEvent *lastConfigureEvent; // Used for replying to MoveResizeWindow
    Bool windowProtocols[UNITY_X11_MAX_WIN_PROTOCOLS];
 
    Bool isRelevant; // Whether the window is relayed through the window tracker
@@ -353,20 +308,6 @@ struct UnityPlatformWindow {
    Bool isViewable;
    Bool wasViewable;
    Bool wantInputFocus;
-   /**
-    * Indicates we wish to force a value of _NET_CURRENT_DESKTOP across XUnmapWindow.
-    *
-    * When we unmap a window, its _NET_CURRENT_DESKTOP property is cleared.  There are
-    * some circumstances in which this behavior is undesirable.  (E.g., once we hide
-    * the KDE taskbar, it will only appear on desktop #0 after exiting Unity, as KDE
-    * doesn't automatically remap panel/dock windows as sticky windows.)
-    *
-    * Setting this flag (in combination with setting onUnmapDesktopNumber) causes
-    * us to reset the _NET_CURRENT_DESKTOP property in response to receiving its
-    * PropertyDelete.  If a PropertyNewValue arrives before the -Delete, we'll query
-    * and record the new value into onUnmapDesktopNumber (if the property exists).
-    */
-   Bool wantSetDesktopNumberOnUnmap;
 
    /*
     * Mini state-machine:
@@ -383,38 +324,6 @@ struct UnityPlatformWindow {
    Bool isHidden;
    Bool isMinimized;
    Bool isMaximized;
-
-   /*
-    * When a client window is reparented by a window manager, a ReparentNotify event
-    * is generated.  We're interested in this event, because it tells us that the
-    * relationship between this UnityPlatformWindow and its top-level and client
-    * windows has changed.
-    *
-    * Per ICCCM §4.1.3.1, we identify the client window as the window within a
-    * tree possessing the WM_STATE property.  Unfortunately, this property may
-    * not yet be set when we receive ReparentNotify.  If that's the case, this flag
-    * is raised such that once we do see a WM_STATE change related to one of
-    * our windows (via a PropertyNotify event), we may act accordingly.
-    */
-   Bool waitingForWmState;
-
-   /*
-    * It isn't safe to delete a UnityPlatformWindow object in the guts of the
-    * event processing code.  (Example:  processing a DestroyNotify event or
-    * learning that a window is irrelevant to Unity/X11.)  Unfortunately, other
-    * parts of this codebase don't use the ref counting scheme correctly, so
-    * this hack will simply mark a UPW for deletion, and the outermost event
-    * entry point will take care of deleting if set.
-    *
-    * XXX Ditch this flag and instead make use of the reference counting
-    * bits.
-    */
-   Bool deleteWhenSafe;
-
-   /*
-    * See wm-spec::_NET_FRAME_EXTENTS.
-    */
-   uint32 frameExtents[4];
 };
 
 /*
@@ -437,23 +346,26 @@ Bool UPWindow_ProtocolSupported(const UnityPlatform *up,
 UnityPlatformWindow *UPWindow_Lookup(UnityPlatform *up, Window window);
 void UPWindow_SetUserTime(UnityPlatform *up,
                           UnityPlatformWindow *upw);
-void UPWindow_SetEWMHDesktop(UnityPlatform *up,
-                             UnityPlatformWindow *upw,
-                             uint32 ewmhDesktopId);
 
 /*
  * Implemented by unityPlatformX11.c
  */
 Bool UnityPlatformWMProtocolSupported(UnityPlatform *up, UnityX11WMProtocol proto);
 Bool UnityPlatformIsRootWindow(UnityPlatform *up, Window window);
+void UnityPlatformSendPendingUpdates(UnityPlatform *up, int flags);
 uint32 UnityX11GetCurrentDesktop(UnityPlatform *up);
 void UnityX11SetCurrentDesktop(UnityPlatform *up, uint32 currentDesktop);
 Time UnityPlatformGetServerTime(UnityPlatform *up);
 
-#define UnityPlatformProcessMainLoop() g_main_context_iteration(NULL, TRUE)
+#if GTK_MAJOR_VERSION >= 2
+#   define UnityPlatformProcessMainLoop() g_main_context_iteration(NULL, TRUE)
+#else
+#   define UnityPlatformProcessMainLoop() gtk_main_iteration()
+#endif
 
 int UnityPlatformGetErrorCount(UnityPlatform *up);
 void UnityPlatformResetErrorCount(UnityPlatform *up);
+void UnityPlatformDumpUpdate(UnityPlatform *up);
 Bool UnityPlatformSetTaskbarVisible(UnityPlatform *up, Bool currentSetting);
 void UnityPlatformSendClientMessage(UnityPlatform *up, Window destWindow,
 				    Window w, Atom messageType,
@@ -465,22 +377,4 @@ size_t UnityPlatformGetNumVirtualDesktops(UnityPlatform *up);
 void UnityPlatformGetVirtualDesktopLayout(UnityPlatform *up, Atom *layoutData);
 const char *UnityPlatformGetEventString(UnityPlatform *up, int type);
 
-gboolean UnityX11HandleEvents(gpointer data);
-
-
-/*
- * Implemented in x11Event.c.
- */
-
-void UnityX11EventEstablishSource(UnityPlatform *up);
-void UnityX11EventTeardownSource(UnityPlatform *up);
-
-
-/*
- * Implemented in x11Util.c.
- */
-
-Bool UnityX11Util_IsWindowDecorationWidget(UnityPlatform *up,
-                                           Window operand);
-
 #endif
