@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2008 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -102,7 +102,6 @@ CPUIDQuery;
 #define CPUID_CACHED_LEVELS                     \
    CPUIDLEVEL(TRUE,  0,  0)                     \
    CPUIDLEVEL(TRUE,  1,  1)                     \
-   CPUIDLEVEL(FALSE, 5,  5)                     \
    CPUIDLEVEL(FALSE,400, 0x40000000)            \
    CPUIDLEVEL(FALSE,410, 0x40000010)            \
    CPUIDLEVEL(FALSE, 80, 0x80000000)            \
@@ -112,11 +111,11 @@ CPUIDQuery;
 
 #define CPUID_UNCACHED_LEVELS                   \
    CPUIDLEVEL(FALSE, 4, 4)                      \
+   CPUIDLEVEL(FALSE, 5, 5)                      \
    CPUIDLEVEL(FALSE, 6, 6)                      \
    CPUIDLEVEL(FALSE, A, 0xA)                    \
-   CPUIDLEVEL(FALSE, B, 0xB)                    \
    CPUIDLEVEL(FALSE, 86, 0x80000006)            \
-   CPUIDLEVEL(FALSE, 87, 0x80000007)
+   CPUIDLEVEL(FALSE, 87, 0x80000007)            \
 
 #define CPUID_ALL_LEVELS                        \
    CPUID_CACHED_LEVELS                          \
@@ -157,19 +156,17 @@ typedef enum {
    CPUID_VENDOR_INTEL,
    CPUID_VENDOR_AMD,
    CPUID_VENDOR_CYRIX,
-   CPUID_VENDOR_VIA,
    CPUID_NUM_VENDORS
 } CpuidVendors;
 
 #define CPUID_INTEL_VENDOR_STRING       "GenuntelineI"
 #define CPUID_AMD_VENDOR_STRING         "AuthcAMDenti"
 #define CPUID_CYRIX_VENDOR_STRING       "CyriteadxIns"
-#define CPUID_VIA_VENDOR_STRING         "CentaulsaurH"
-#define CPUID_HYPERV_HYPERVISOR_VENDOR_STRING  "Microsoft Hv"
 #define CPUID_INTEL_VENDOR_STRING_FIXED "GenuineIntel"
 #define CPUID_AMD_VENDOR_STRING_FIXED   "AuthenticAMD"
 #define CPUID_CYRIX_VENDOR_STRING_FIXED "CyrixInstead"
-#define CPUID_VIA_VENDOR_STRING_FIXED   "CentaurHauls"
+
+#define CPUID_HYPERV_HYPERVISOR_VENDOR_STRING  "Microsoft Hv"
 
 /*
  * FIELDDEF can be defined to process the CPUID information provided
@@ -264,12 +261,11 @@ FIELDDEF(  1, EBX, COMMON,  8,  8, CLFL_SIZE,           ANY, IGNORE, 0, FALSE)
 FIELDDEFA( 1, EBX, COMMON, 16,  8, LCPU_COUNT,          ANY, IGNORE, 0, FALSE, LCPU_COUNT) \
 FIELDDEFA( 1, EBX, COMMON, 24,  8, APICID,              ANY, IGNORE, 0, FALSE, APICID)    \
 FLAGDEFA(  1, ECX, COMMON, 0,   1, SSE3,                YES, HOST,   0, TRUE,  SSE3)      \
-FLAGDEFA(  1, ECX, INTEL,  1,   1, PCLMULQDQ,           YES, HOST,   0, TRUE, PCLMULQDQ)  \
 FLAGDEF(   1, ECX, INTEL,  2,   1, NDA2,                NO,  MASK,   0, FALSE)            \
-FLAGDEFA(  1, ECX, COMMON, 3,   1, MWAIT,               ANY, MASK,   0, FALSE, MWAIT)     \
+FLAGDEFA(  1, ECX, COMMON, 3,   1, MWAIT,               NO,  MASK,   0, FALSE, MWAIT)     \
 FLAGDEFA(  1, ECX, INTEL,  4,   1, DSCPL,               NO,  MASK,   0, FALSE, DSCPL)     \
 FLAGDEFA(  1, ECX, INTEL,  5,   1, VMX,                 NO,  MASK,   0, FALSE, VMX)       \
-FLAGDEF(   1, ECX, INTEL,  6,   1, SMX,                 NO,  MASK,   0, FALSE) \
+FLAGDEF(   1, ECX, INTEL,  6,   1, SMX,                 NO,  MASK,   0, FALSE)            \
 FLAGDEF(   1, ECX, INTEL,  7,   1, EST,                 NO,  MASK,   0, FALSE)            \
 FLAGDEF(   1, ECX, INTEL,  8,   1, TM2,                 NO,  MASK,   0, FALSE)            \
 FLAGDEFA(  1, ECX, COMMON, 9,   1, SSSE3,               YES, HOST,   0, TRUE,  SSSE3)     \
@@ -281,10 +277,9 @@ FLAGDEF(   1, ECX, INTEL,  18,  1, DCA,                 NO,  MASK,   0, FALSE)
 FLAGDEFA(  1, ECX, INTEL,  19,  1, SSE41,               YES, HOST,   0, TRUE,  SSE41)     \
 FLAGDEFA(  1, ECX, INTEL,  20,  1, SSE42,               YES, HOST,   0, TRUE,  SSE42)     \
 FLAGDEF(   1, ECX, INTEL,  21,  1, X2APIC,              NO,  MASK,   0, FALSE)            \
-FLAGDEFA(  1, ECX, INTEL,  22,  1, MOVBE,               YES, HOST,   0, TRUE,  MOVBE)     \
+FLAGDEF(   1, ECX, INTEL,  22,  1, MOVBE,               NO,  RSVD,   0, TRUE)             \
 FLAGDEFA(  1, ECX, COMMON, 23,  1, POPCNT,              YES, HOST,   0, TRUE,  POPCNT)    \
 FLAGDEF(   1, ECX, INTEL,  24,  1, ULE,                 NO,  RSVD,   0, TRUE)             \
-FLAGDEFA(  1, ECX, INTEL,  25,  1, AES,                 YES, HOST,   0, TRUE, AES)        \
 FLAGDEF(   1, ECX, INTEL,  26,  1, XSAVE,               NO,  MASK,   0, FALSE)            \
 FLAGDEF(   1, ECX, INTEL,  27,  1, OSXSAVE,             NO,  RSVD,   0, TRUE)             \
 FLAGDEFA(  1, ECX, COMMON, 31,  1, HYPERVISOR,          ANY, IGNORE, 0, FALSE, HYPERVISOR)\
@@ -323,23 +318,18 @@ FLAGDEF(   1, EDX, INTEL,  31,  1, PBE,                 NO,  MASK,   0, FALSE)
 #define CPUID_FIELD_DATA_LEVEL_4                                               \
 FIELDDEF(  4, EAX, INTEL,   0,  5, CACHE_TYPE,          NA,  IGNORE, 0, FALSE)            \
 FIELDDEF(  4, EAX, INTEL,   5,  3, CACHE_LEVEL,         NA,  IGNORE, 0, FALSE)            \
-FLAGDEF(   4, EAX, INTEL,   8,  1, CACHE_SELF_INIT,     NA,  IGNORE, 0, FALSE)            \
-FLAGDEF(   4, EAX, INTEL,   9,  1, CACHE_FULLY_ASSOC,   NA,  IGNORE, 0, FALSE)            \
 FIELDDEF(  4, EAX, INTEL,  14, 12, CACHE_NUMHT_SHARING, NA,  IGNORE, 0, FALSE)            \
 FIELDDEFA( 4, EAX, INTEL,  26,  6, CORE_COUNT,          NA,  IGNORE, 0, FALSE, INTEL_CORE_COUNT)  \
 FIELDDEF(  4, EBX, INTEL,   0, 12, CACHE_LINE,          NA,  IGNORE, 0, FALSE)            \
 FIELDDEF(  4, EBX, INTEL,  12, 10, CACHE_PART,          NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  4, EBX, INTEL,  22, 10, CACHE_WAYS,          NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  4, ECX, INTEL,   0, 32, CACHE_SETS,          NA,  IGNORE, 0, FALSE)            \
-FLAGDEF(   4, EDX, INTEL,   0,  1, CACHE_WBINVD_NOT_GUARANTEED, NA,  IGNORE, 0, FALSE)    \
-FLAGDEF(   4, EDX, INTEL,   1,  1, CACHE_IS_INCLUSIVE,  NA,  IGNORE, 0, FALSE)
+FIELDDEF(  4, EBX, INTEL,  22, 10, CACHE_WAYS,          NA,  IGNORE, 0, FALSE)
 
 /*     LEVEL, REG, VENDOR, POS, SIZE, NAME,       MON SUPP, MASK TYPE, SET TO, CPL3, [FUNC] */
 #define CPUID_FIELD_DATA_LEVEL_5                                           \
-FIELDDEFA( 5, EAX, COMMON,  0, 16, MWAIT_MIN_SIZE,      NA,  IGNORE, 0, FALSE, MWAIT_MIN_SIZE) \
-FIELDDEFA( 5, EBX, COMMON,  0, 16, MWAIT_MAX_SIZE,      NA,  IGNORE, 0, FALSE, MWAIT_MAX_SIZE) \
+FIELDDEF(  5, EAX, COMMON,  0, 16, MWAIT_MIN_SIZE,      NA,  IGNORE, 0, FALSE) \
+FIELDDEF(  5, EBX, COMMON,  0, 16, MWAIT_MAX_SIZE,      NA,  IGNORE, 0, FALSE) \
 FLAGDEF(   5, ECX, COMMON,  0,  1, MWAIT_EXTENSIONS,    NA,  IGNORE, 0, FALSE) \
-FLAGDEFA(  5, ECX, COMMON,  1,  1, MWAIT_INTR_BREAK,    NA,  IGNORE, 0, FALSE, MWAIT_INTR_BREAK) \
+FLAGDEF(   5, ECX, COMMON,  1,  1, MWAIT_INTR_BREAK,    NA,  IGNORE, 0, FALSE) \
 FIELDDEF(  5, EDX, INTEL,   0,  4, MWAIT_C0_SUBSTATE,   NA,  IGNORE, 0, FALSE) \
 FIELDDEF(  5, EDX, INTEL,   4,  4, MWAIT_C1_SUBSTATE,   NA,  IGNORE, 0, FALSE) \
 FIELDDEF(  5, EDX, INTEL,   8,  4, MWAIT_C2_SUBSTATE,   NA,  IGNORE, 0, FALSE) \
@@ -365,17 +355,7 @@ FLAGDEF(   A, EBX, INTEL,   2,  1, PMC_REF_CYCLES,      NA,  IGNORE, 0, FALSE)
 FLAGDEF(   A, EBX, INTEL,   3,  1, PMC_LAST_LVL_CREF,   NA,  IGNORE, 0, FALSE)            \
 FLAGDEF(   A, EBX, INTEL,   4,  1, PMC_LAST_LVL_CMISS,  NA,  IGNORE, 0, FALSE)            \
 FLAGDEF(   A, EBX, INTEL,   5,  1, PMC_BR_INST_RETIRED, NA,  IGNORE, 0, FALSE)            \
-FLAGDEF(   A, EBX, INTEL,   6,  1, PMC_BR_MISS_RETIRED, NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  A, EDX, INTEL,   0,  5, PMC_FIXED_NUM,       NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  A, EDX, INTEL,   5,  8, PMC_FIXED_SIZE,      NA,  IGNORE, 0, FALSE)
-
-/*    LEVEL, REG, VENDOR, POS, SIZE, NAME,       MON SUPP, MASK TYPE, SET TO, CPL3, [FUNC] */
-#define CPUID_FIELD_DATA_LEVEL_B                                               \
-FIELDDEF(  B, EAX, INTEL,   0,  5, MASK_WIDTH,          NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  B, EBX, INTEL,   0, 16, CPUS_SHARING_LEVEL,  NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  B, ECX, INTEL,   0,  8, LEVEL_NUMBER,        NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  B, ECX, INTEL,   8,  8, LEVEL_TYPE,          NA,  IGNORE, 0, FALSE)            \
-FIELDDEF(  B, EDX, INTEL,   0, 32, X2APIC_ID,           NA,  IGNORE, 0, FALSE)
+FLAGDEF(   A, EBX, INTEL,   6,  1, PMC_BR_MISS_RETIRED, NA,  IGNORE, 0, FALSE)
 
 /*    LEVEL, REG, VENDOR, POS, SIZE, NAME,       MON SUPP, MASK TYPE, SET TO, CPL3, [FUNC] */
 #define CPUID_FIELD_DATA_LEVEL_80                                              \
@@ -432,7 +412,7 @@ FLAGDEFA( 81, EDX, AMD,    22,  1, MMXEXT,              YES, HOST,   0, TRUE,  M
 FLAGDEF(  81, EDX, AMD,    23,  1, MMX,                 YES, HOST,   0, TRUE)             \
 FLAGDEF(  81, EDX, AMD,    24,  1, FXSAVE,              YES, HOST,   0, TRUE)             \
 FLAGDEFA( 81, EDX, AMD,    25,  1, FFXSR,               YES, HOST,   0, FALSE, FFXSR)     \
-FLAGDEF(  81, EDX, COMMON, 26,  1, PDPE1GB,             NO,  MASK,   0, FALSE)            \
+FLAGDEF(  81, EDX, AMD,    26,  1, PDPE1GB,             NO,  MASK,   0, FALSE)            \
 FLAGDEFA( 81, EDX, COMMON, 27,  1, RDTSCP,              YES, HOST,   0, TRUE,  RDTSCP)    \
 FLAGDEFA( 81, EDX, COMMON, 29,  1, LM,                  YES, TEST,   1, FALSE, LM) \
 FLAGDEFA( 81, EDX, AMD,    30,  1, 3DNOWPLUS,           YES, HOST,   0, TRUE,  3DNOWPLUS) \
@@ -456,11 +436,11 @@ FLAGDEF(  87, EDX, AMD,     4,  1, TM,                  NA,  IGNORE, 0, FALSE)
 FLAGDEF(  87, EDX, AMD,     5,  1, STC,                 NA,  IGNORE, 0, FALSE)            \
 FLAGDEF(  87, EDX, AMD,     6,  1, 100MHZSTEPS,         NA,  IGNORE, 0, FALSE)            \
 FLAGDEF(  87, EDX, AMD,     7,  1, HWPSTATE,            NA,  IGNORE, 0, FALSE)            \
-FLAGDEF(  87, EDX, COMMON,  8,  1, TSC_INVARIANT,       NA,  IGNORE, 0, FALSE)            \
+FLAGDEF(  87, EDX, AMD,     8,  1, TSC_INVARIANT,       NA,  IGNORE, 0, FALSE)            \
 FIELDDEFA(88, EAX, COMMON,  0,  8, PHYSBITS,            NA,  IGNORE, 0, FALSE, PHYS_BITS) \
 FIELDDEFA(88, EAX, COMMON,  8,  8, VIRTBITS,            NA,  IGNORE, 0, FALSE, VIRT_BITS) \
 FIELDDEFA(88, ECX, AMD,     0,  8, CORE_COUNT,          NA,  IGNORE, 0, FALSE, AMD_CORE_COUNT) \
-FIELDDEFA(88, ECX, AMD,    12,  4, APICID_COREID_SIZE,  NA,  IGNORE, 0, FALSE, AMD_APICID_COREID_SIZE) \
+FIELDDEF( 88, ECX, AMD,    12,  4, APICID_COREID_SIZE,  NA,  IGNORE, 0, FALSE)            \
 FIELDDEFA(8A, EAX, AMD,     0,  8, SVM_REVISION,        NO,  MASK,   0, FALSE, SVM_REVISION) \
 FLAGDEF(  8A, EAX, AMD,     8,  1, SVM_HYPERVISOR,      NO,  MASK,   0, FALSE)            \
 FIELDDEF( 8A, EAX, AMD,     9, 23, SVMEAX_RSVD,         NO,  MASK,   0, FALSE)            \
@@ -472,7 +452,6 @@ FLAGDEF(  8A, EDX, AMD,     2,  1, SVM_LOCK,            NO,  MASK,   0, FALSE)
 FLAGDEF(  8A, EDX, AMD,     3,  1, SVM_NRIP,            NO,  MASK,   0, FALSE)            \
 FIELDDEF( 8A, EDX, AMD,     4, 28, SVMEDX_RSVD,         NO,  MASK,   0, FALSE)
 
-
 #define CPUID_FIELD_DATA                                              \
    CPUID_FIELD_DATA_LEVEL_0                                           \
    CPUID_FIELD_DATA_LEVEL_1                                           \
@@ -480,12 +459,10 @@ FIELDDEF( 8A, EDX, AMD,     4, 28, SVMEDX_RSVD,         NO,  MASK,   0, FALSE)
    CPUID_FIELD_DATA_LEVEL_5                                           \
    CPUID_FIELD_DATA_LEVEL_6                                           \
    CPUID_FIELD_DATA_LEVEL_A                                           \
-   CPUID_FIELD_DATA_LEVEL_B                                           \
    CPUID_FIELD_DATA_LEVEL_80                                          \
    CPUID_FIELD_DATA_LEVEL_81                                          \
    CPUID_FIELD_DATA_LEVEL_8x
 
-
 /*
  * Define all field and flag values as an enum.  The result is a full
  * set of values taken from the table above in the form:
@@ -609,7 +586,6 @@ FIELD_FUNC(PA_BITS,          CPUID_COMMON_ID88EAX_PHYSBITS)
 FIELD_FUNC(VIRT_BITS,        CPUID_COMMON_ID88EAX_VIRTBITS)
 FIELD_FUNC(SVM_REVISION,     CPUID_AMD_ID8AEAX_SVM_REVISION)
 FIELD_FUNC(SVM_N_ASIDS,      CPUID_AMD_ID8AEBX_SVM_N_ASIDS)
-FIELD_FUNC(CACHE_TYPE,       CPUID_INTEL_ID4EAX_CACHE_TYPE)
 FIELD_FUNC(INTEL_CORE_COUNT, CPUID_INTEL_ID4EAX_CORE_COUNT)
 FIELD_FUNC(AMD_CORE_COUNT,   CPUID_AMD_ID88ECX_CORE_COUNT)
 FIELD_FUNC(AMD_APICID_COREID_SIZE, CPUID_AMD_ID88ECX_APICID_COREID_SIZE)
@@ -622,11 +598,6 @@ FIELD_FUNC(MWAIT_C1_SUBSTATE, CPUID_INTEL_ID5EDX_MWAIT_C1_SUBSTATE)
 FIELD_FUNC(MWAIT_C2_SUBSTATE, CPUID_INTEL_ID5EDX_MWAIT_C2_SUBSTATE)
 FIELD_FUNC(MWAIT_C3_SUBSTATE, CPUID_INTEL_ID5EDX_MWAIT_C3_SUBSTATE)
 FIELD_FUNC(MWAIT_C4_SUBSTATE, CPUID_INTEL_ID5EDX_MWAIT_C4_SUBSTATE)
-FIELD_FUNC(TOPOLOGY_MASK_WIDTH,         CPUID_INTEL_IDBEAX_MASK_WIDTH)
-FIELD_FUNC(TOPOLOGY_CPUS_SHARING_LEVEL, CPUID_INTEL_IDBEBX_CPUS_SHARING_LEVEL)
-FIELD_FUNC(TOPOLOGY_LEVEL_NUMBER,       CPUID_INTEL_IDBECX_LEVEL_NUMBER)
-FIELD_FUNC(TOPOLOGY_LEVEL_TYPE,         CPUID_INTEL_IDBECX_LEVEL_TYPE)
-FIELD_FUNC(TOPOLOGY_X2APIC_ID,          CPUID_INTEL_IDBEDX_X2APIC_ID)
 #undef FIELD_FUNC
 
 
@@ -635,8 +606,6 @@ FIELD_FUNC(TOPOLOGY_X2APIC_ID,          CPUID_INTEL_IDBEDX_X2APIC_ID)
  * macros/functions for reading cpuid fields.
  */
 
-#define CPUID_FAMILY_EXTENDED 15
-
 /* Effective Intel CPU Families */
 #define CPUID_FAMILY_486      4
 #define CPUID_FAMILY_P5       5
@@ -651,6 +620,7 @@ FIELD_FUNC(TOPOLOGY_X2APIC_ID,          CPUID_INTEL_IDBEDX_X2APIC_ID)
 #define CPUID_FAMILY_K8       15
 #define CPUID_FAMILY_K8L      16
 #define CPUID_FAMILY_K8MOBILE 17
+#define CPUID_FAMILY_EXTENDED 15
 
 /* Intel model information */
 #define CPUID_MODEL_PPRO       1
@@ -665,52 +635,11 @@ FIELD_FUNC(TOPOLOGY_X2APIC_ID,          CPUID_INTEL_IDBEDX_X2APIC_ID)
 #define CPUID_MODEL_NEHALEM_1A 0x1a  // Nehalem / Gainestown
 #define CPUID_MODEL_ATOM_1C    0x1c  // Silverthorne / Diamondville
 #define CPUID_MODEL_CORE_1D    0x1d  // Dunnington
-#define CPUID_MODEL_NEHALEM_1E 0x1e  // Lynnfield
 
 #define CPUID_MODEL_PIII_07    7
 #define CPUID_MODEL_PIII_08    8
 #define CPUID_MODEL_PIII_0A    10
 
-/* AMD model information */
-#define CPUID_MODEL_BARCELONA_02 0x02 // Barcelona (both Opteron & Phenom kind)
-
-/*
- *----------------------------------------------------------------------
- *
- * CPUID_IsVendor{AMD,Intel} --
- *
- *      Determines if the vendor string in cpuid id0 is from {AMD,Intel}.
- *
- * Results:
- *      True iff vendor string is CPUID_{AMD,INTEL}_VENDOR_STRING
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-static INLINE Bool
-CPUID_IsRawVendor(CPUIDRegs *id0, const char* vendor)
-{
-   // hard to get strcmp() in some environments, so do it in the raw
-   return (id0->ebx == *(const uint32 *) (vendor + 0) &&
-           id0->ecx == *(const uint32 *) (vendor + 4) &&
-           id0->edx == *(const uint32 *) (vendor + 8));   
-}
-
-static INLINE Bool
-CPUID_IsVendorAMD(CPUIDRegs *id0)
-{
-   return CPUID_IsRawVendor(id0, CPUID_AMD_VENDOR_STRING);
-}
-
-static INLINE Bool
-CPUID_IsVendorIntel(CPUIDRegs *id0)
-{
-   return CPUID_IsRawVendor(id0, CPUID_INTEL_VENDOR_STRING);
-}
-
-
 static INLINE uint32
 CPUID_EFFECTIVE_FAMILY(uint32 v) /* %eax from CPUID with %eax=1. */
 {
@@ -785,21 +714,19 @@ CPUID_UARCH_IS_CORE(uint32 v) // IN: %eax from CPUID with %eax=1.
            model == CPUID_MODEL_CORE_1D);
 }
 
+
 /*
- * Intel Nehalem processors are: Nehalem, Gainestown, Lynnfield, Clarkdale.
+ * Intel Nehalem processors are: Nehalem, Gainestown.
  */
 static INLINE Bool
 CPUID_UARCH_IS_NEHALEM(uint32 v) // IN: %eax from CPUID with %eax=1.
 {
    /* Assumes the CPU manufacturer is Intel. */
-   uint32 effectiveModel = CPUID_EFFECTIVE_MODEL(v);
-
    return CPUID_FAMILY_IS_P6(v) &&
-          (
-           effectiveModel == CPUID_MODEL_NEHALEM_1E ||
-           effectiveModel == CPUID_MODEL_NEHALEM_1A);
+          CPUID_EFFECTIVE_MODEL(v) == CPUID_MODEL_NEHALEM_1A;
 }
 
+
 static INLINE Bool
 CPUID_FAMILY_IS_K7(uint32 _eax)
 {
@@ -847,17 +774,6 @@ CPUID_FAMILY_IS_K8STAR(uint32 _eax)
           CPUID_FAMILY_IS_K8MOBILE(_eax);
 }
 
-/*
- * AMD Barcelona (of either Opteron or Phenom kind).
- */
-static INLINE Bool
-CPUID_MODEL_IS_BARCELONA(uint32 v) // IN: %eax from CPUID with %eax=1.
-{
-   /* Assumes the CPU manufacturer is AMD. */
-   return CPUID_FAMILY_IS_K8L(v) &&
-          CPUID_EFFECTIVE_MODEL(v) == CPUID_MODEL_BARCELONA_02;
-}
-
 
 #define CPUID_TYPE_PRIMARY     0
 #define CPUID_TYPE_OVERDRIVE   1
@@ -871,31 +787,28 @@ CPUID_MODEL_IS_BARCELONA(uint32 v) // IN: %eax from CPUID with %eax=1.
 #define CPUID_INTEL_ID4EAX_CACHE_SELF_INIT      0x00000100
 #define CPUID_INTEL_ID4EAX_CACHE_FULLY_ASSOC    0x00000200
 
-#define CPUID_INTEL_IDBECX_LEVEL_TYPE_INVALID   0
-#define CPUID_INTEL_IDBECX_LEVEL_TYPE_SMT       1
-#define CPUID_INTEL_IDBECX_LEVEL_TYPE_CORE      2
 
 /*
  * On AMD chips before Opteron and Intel chips before P4 model 3,
  * WRMSR(TSC) clears the upper half of the TSC instead of using %edx.
  */
 static INLINE Bool
-CPUID_FullyWritableTSC(CpuidVendors vendor, // IN
-                       uint32 v)      // IN: %eax from CPUID with %eax=1.
+CPUID_FullyWritableTSC(Bool isIntel, // IN
+                       uint32 v)     // IN: %eax from CPUID with %eax=1.
 {
    /*
     * Returns FALSE if:
     *   - Intel && P6 (pre-core) or
     *   - Intel && P4 (model < 3) or
-    *   - AMD && pre-K8 Opteron
-    * Otherwise, returns TRUE (including for Via Nano).
+    *   - !Intel && pre-K8 Opteron
+    * Otherwise, returns TRUE.
     */
-   return !((vendor == CPUID_VENDOR_INTEL &&
+   return !((isIntel &&
              ((CPUID_FAMILY_IS_P6(v) &&
                CPUID_EFFECTIVE_MODEL(v) < CPUID_MODEL_PM_0E) ||
               (CPUID_FAMILY_IS_PENTIUM4(v) &&
                CPUID_EFFECTIVE_MODEL(v) < 3))) ||
-            (vendor == CPUID_VENDOR_AMD &&
+            (!isIntel &&
              CPUID_FAMILY(v) < CPUID_FAMILY_K8));
 }
 
@@ -924,7 +837,10 @@ CPUID_ID0RequiresFence(CPUIDRegs *id0)
    if (id0->eax == 0) {
       return FALSE;
    }
-   return CPUID_IsVendorAMD(id0);
+   // hard to get strcmp() in some environments, so do it in the raw
+   return (id0->ebx == *(uint32 *) (CPUID_AMD_VENDOR_STRING + 0) &&
+           id0->ecx == *(uint32 *) (CPUID_AMD_VENDOR_STRING + 4) &&
+           id0->edx == *(uint32 *) (CPUID_AMD_VENDOR_STRING + 8));
 }
 
 static INLINE Bool
