@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -38,7 +38,13 @@
  */
 
 int LOGLEVEL_THRESHOLD = 4;
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 9)
 module_param(LOGLEVEL_THRESHOLD, int, 0444);
+#else
+MODULE_PARM(LOGLEVEL_THRESHOLD, "i");
+#endif
+
 MODULE_PARM_DESC(LOGLEVEL_THRESHOLD, "Set verbosity (0 means no log, 10 means very verbose, 4 is default)");
 #endif
 
@@ -47,14 +53,6 @@ MODULE_AUTHOR("VMware, Inc.");
 MODULE_DESCRIPTION("VMware Host/Guest File System");
 MODULE_VERSION(VMHGFS_DRIVER_VERSION_STRING);
 MODULE_LICENSE("GPL v2");
-MODULE_ALIAS("vmware_vmhgfs");
-/*
- * Starting with SLE10sp2, Novell requires that IHVs sign a support agreement
- * with them and mark their kernel modules as externally supported via a
- * change to the module header. If this isn't done, the module will not load
- * by default (i.e., neither mkinitrd nor modprobe will accept it).
- */
-MODULE_INFO(supported, "external");
 
 
 /*
